@@ -864,10 +864,10 @@ table_insert_row (ETable *table, void *data)
 
 	model = table->model;
 
-	e_table_memory_freeze (E_TABLE_MEMORY (model));
+	/* e_table_memory_freeze (E_TABLE_MEMORY (model)); */
 	e_table_memory_insert (E_TABLE_MEMORY (model), -1, data);
 	e_table_set_cursor_row (table, e_table_model_row_count (model) - 1);
-	e_table_memory_thaw (E_TABLE_MEMORY (model));
+	/* e_table_memory_thaw (E_TABLE_MEMORY (model)); */
 }
 
 /* --- */
